@@ -1,6 +1,6 @@
 """ Time transformations
 
-This handkes all the wibbly wobbly timey wimey stuff.
+This handles all the wibbly wobbly timey wimey stuff.
 Such as easy conversions between different time systems.
 Supported systems: GPS, UTC, GMST, LST, JD and MJD.
 
@@ -134,7 +134,7 @@ def datetime_modifiedjd(dt):
 
 
 def juliandate_to_gmst(juliandate):
-    """Convert a datetime object in UTC time to Greenwich Mean Sidereal Time
+    """Convert a Julian Date to Greenwich Mean Sidereal Time
 
     :param juliandate: Julian Date
     :return: decimal hours in GMST
@@ -222,14 +222,14 @@ def juliandate_to_utc(juliandate):
     if month > 2.5:
         year = D - 4716
     else:
-        year = D - 4715 # year
+        year = D - 4715
     year = int(year)
 
     day_frac, day = math.modf(day)
 
     day = int(day)
     date = datetime.date(year, month, day)
-    hours = day_frac * 24 # fractional part of day * 24 hours
+    hours = day_frac * 24  # fractional part of day * 24 hours
 
     time = decimal_to_time(hours)
 
